@@ -6,6 +6,22 @@
 set -e
 set -u
 
+cmdcheck() { command -v "$1" >/dev/null 2>&1; }
+
+assert_commands() {
+    local missing=()
+    local cmd=
+
+    for cmd in "$@"; do
+        cmdcheck $cmd || missing+=($cmd)
+    done
+
+    [ ${#missing[@]} -eq 0 ] && return 0
+
+    echo "Missing command(s): ${missing[@]}" >&2
+    echo "Please install it and retry." >&2
+    exit 1
+}
 
 ## repology backend - only gives email addresses though
 
@@ -73,21 +89,20 @@ FEDORA_URL=https://src.fedoraproject.org
 
 http_get_fedora_maintainer() {
 
-    # Changelog headline, as found in the .spec file:
+    # Changelog headline, as found in the changelog file:
     # * Thu Jul 22 2021 Fedora Release Engineering <releng@fedoraproject.org> - 0.6.3-2
     # * Thu Feb 06 22:29:07 CET 2020 Robert-André Mauchin <zebob.m@gmail.com> - 0.5.1-1
 
-    local spec_url=
-    local spec=
+    local changelog_url=
     local changelog=
     local entries=
     local entry=
     local maint=
 
-    spec_url=$FEDORA_URL/rpms/goodvibes/raw/rawhide/f/goodvibes.spec
-    spec=$(wget -q -O- "$spec_url")
-    changelog=$(echo "$spec" | sed -n '/%changelog/,$ p')
+    changelog_url=$FEDORA_URL/rpms/goodvibes/raw/rawhide/f/changelog
+    changelog=$(wget -q -O- "$changelog_url")
     entries=$(echo "$changelog" | grep '^*')
+
     while read -r entry; do
         maint=$(echo "$entry" | sed 's/^.*20[2-9][0-9]  *//' | rev | sed 's/^.* -  *//' | rev)
         case "$maint" in
@@ -98,40 +113,39 @@ http_get_fedora_maintainer() {
     done <<< $entries
 }
 
-OPENSUSE_URL=https://download.opensuse.org
+NIXOS_URL=https://raw.githubusercontent.com/NixOS/nixpkgs
+
+http_get_nixos_maintainer() {
+
+    local recipe=
+    local maint=
+
+    recipe=$(wget -q -O- "$NIXOS_URL/nixos-unstable/pkgs/applications/audio/goodvibes/default.nix")
+    maint=$(echo "$recipe" | grep 'maintainers = ' | sed -E 's/.*\[ (.*) \].*/\1/')
+
+    echo "$maint"
+}
+
+OPENSUSE_URL=https://build.opensuse.org
 
 http_get_opensuse_maintainer() {
 
-    # Changelog headline, as displayed per the rpm command:
-    # * Thu Sep 26 2019 Alexei Podvalsky <avvissu@yandex.by>
+    # Changelog headline, as found in the .changes file:
+    # Sat Nov 11 14:27:58 UTC 2023 - Andrea Manzini <andrea.manzini@suse.com>
+    #
+    # Sloppy as long as I don't know of a way to get the raw .changes file,
+    # instead of getting a HTML page and then extracting the .changes file
+    # out of the HTML, and the converting HTML entities, etc etc...
 
-    local tmpdir=
-    local rpmpkg=
-    local changelog=
-    local latest_entry=
-    local maint
+    local changes_html=
+    local changes=
+    local entries=
+    local maint=
 
-    tmpdir=$(mktemp -d)
-    pushd "$tmpdir" >/dev/null
-    trap "popd >/dev/null && rm -fr $tmpdir" EXIT
-
-    wget \
-        --quiet \
-        --recursive \
-        --level=1 \
-        --no-parent \
-        --no-directories \
-        --accept-regex 'goodvibes-.*\.src\.rpm$' \
-        "$OPENSUSE_URL/repositories/multimedia:/apps/openSUSE_Tumbleweed/src/"
-
-    rpmpkg=$(ls -1 *.rpm)
-    changelog=$(rpm -q --changelog $rpmpkg 2>/dev/null)
-    latest_entry=$(echo "$changelog" | grep '^*' | head -1)
-    maint=$(echo "$latest_entry" | cut -d' ' -f 6-)
-
-    trap - EXIT
-    popd > /dev/null
-    rm -fr "$tmpdir"
+    changes_html=$(wget -q -O- "$OPENSUSE_URL/projects/openSUSE:Factory/packages/goodvibes/files/goodvibes.changes")
+    changes=$(echo "$changes_html" | pup 'pre text{}' | perl -MHTML::Entities -pe 'decode_entities($_);')
+    entries=$(echo "$changes" | grep -E '^(Mon|Tue|Wed|Thu|Fri|Sat|Sun)')
+    maint=$(echo "$entries" | head -1 | sed 's/.* - //')
 
     echo "$maint"
 }
@@ -164,6 +178,8 @@ http_query() {
     [ -n "$maint" ] && echo " * $maint - Debian"
     maint=$(http_get_fedora_maintainer)
     [ -n "$maint" ] && echo " * $maint - Fedora"
+    maint=$(http_get_nixos_maintainer)
+    [ -n "$maint" ] && echo " * $maint - NixOS"
     maint=$(http_get_opensuse_maintainer)
     [ -n "$maint" ] && echo " * $maint - openSUSE"
     maint=$(http_get_void_maintainer)
@@ -180,5 +196,7 @@ strip_email() {
 
 ## main
 
+assert_commands jq pup wget
+
 #repology_query
 http_query | strip_email
