@@ -1,7 +1,7 @@
 /*
  * Goodvibes Radio Player
  *
- * Copyright (C) 2015-2021 Arnaud Rebillout
+ * Copyright (C) 2015-2024 Arnaud Rebillout
  *
  * SPDX-License-Identifier: GPL-3.0-only
  *
@@ -28,10 +28,9 @@
 #include "core/gv-core-enum-types.h"
 #include "core/gv-core-internal.h"
 #include "core/gv-engine.h"
-#include "core/gv-metadata.h"
+#include "core/gv-playback.h"
 #include "core/gv-station-list.h"
 #include "core/gv-station.h"
-#include "core/gv-streaminfo.h"
 
 #include "core/gv-player.h"
 
@@ -50,16 +49,15 @@ enum {
 	PROP_0,
 	/* Construct-only properties */
 	PROP_ENGINE,
+	PROP_PLAYBACK,
 	PROP_STATION_LIST,
 	/* Engine mirrored properties */
-	PROP_STREAMINFO,
-	PROP_METADATA,
 	PROP_VOLUME,
 	PROP_MUTE,
 	PROP_PIPELINE_ENABLED,
 	PROP_PIPELINE_STRING,
 	/* Properties */
-	PROP_PLAYBACK_STATE,
+	PROP_PLAYING,
 	PROP_REPEAT,
 	PROP_SHUFFLE,
 	PROP_AUTOPLAY,
@@ -73,40 +71,21 @@ enum {
 
 static GParamSpec *properties[PROP_N];
 
-/*
- * Signals
- */
-
-enum {
-	SIGNAL_SSL_FAILURE,
-	/* Number of signals */
-	SIGNAL_N
-};
-
-static guint signals[SIGNAL_N];
-
 /*
  * GObject definitions
  */
 
-typedef enum {
-	GV_PLAYER_WISH_TO_STOP,
-	GV_PLAYER_WISH_TO_PLAY,
-} GvPlayerWish;
-
 struct _GvPlayerPrivate {
 	/* Construct-only properties */
 	GvEngine *engine;
+	GvPlayback *playback;
 	GvStationList *station_list;
 	/* Properties */
-	GvPlaybackState state;
+	gboolean playing;
 	gboolean repeat;
 	gboolean shuffle;
 	gboolean autoplay;
-	/* Current station */
 	GvStation *station;
-	/* Wished state */
-	GvPlayerWish wish;
 };
 
 typedef struct _GvPlayerPrivate GvPlayerPrivate;
@@ -123,146 +102,41 @@ static void gv_player_configurable_interface_init(GvConfigurableInterface *iface
 G_DEFINE_TYPE_WITH_CODE(GvPlayer, gv_player, G_TYPE_OBJECT,
 			G_ADD_PRIVATE(GvPlayer)
 			G_IMPLEMENT_INTERFACE(GV_TYPE_CONFIGURABLE,
-					      gv_player_configurable_interface_init)
-			G_IMPLEMENT_INTERFACE(GV_TYPE_ERRORABLE, NULL))
-
-/*
- * Playback
- */
-
-const gchar *
-gv_playback_state_to_string(GvPlaybackState state)
-{
-	const gchar *str;
-
-	switch (state) {
-	case GV_PLAYBACK_STATE_PLAYING:
-		str = _("Playing");
-		break;
-	case GV_PLAYBACK_STATE_CONNECTING:
-		str = _("Connecting…");
-		break;
-	case GV_PLAYBACK_STATE_BUFFERING:
-		str = _("Buffering…");
-		break;
-	case GV_PLAYBACK_STATE_STOPPED:
-	default:
-		str = _("Stopped");
-		break;
-	}
-
-	return str;
-}
+					      gv_player_configurable_interface_init))
 
 /*
  * Signal handlers
  */
 
-static void gv_player_set_playback_state(GvPlayer *self, GvPlaybackState value);
+typedef struct {
+	const gchar *name;
+	guint        id;
+} PropertyMapping;
+
+static const PropertyMapping engine_mappings[] = {
+	{ "volume", PROP_VOLUME },
+	{ "mute", PROP_MUTE },
+	{ "pipeline-enabled", PROP_PIPELINE_ENABLED },
+	{ "pipeline-string", PROP_PIPELINE_STRING },
+	{ NULL, 0 },
+};
 
 static void
-on_station_notify(GvStation *station,
-		  GParamSpec *pspec,
-		  GvPlayer *self)
+on_engine_notify(GvEngine *engine, GParamSpec *pspec, GvPlayer *self)
 {
-	GvPlayerPrivate *priv = self->priv;
-	const gchar *property_name = g_param_spec_get_name(pspec);
-
-	TRACE("%p, %s, %p", station, property_name, self);
-
-	g_assert(station == priv->station);
-
-	if (!g_strcmp0(property_name, "stream-uris")) {
-		DEBUG("Station %p: stream URIs have changed", station);
-
-		/* Check if there are some streams, and start playing if needed */
-		if (gv_station_get_stream_uris(station))
-			if (priv->wish == GV_PLAYER_WISH_TO_PLAY)
-				gv_player_play(self);
-	}
-
-	/* In any case, we notify if something was changed in the station */
-	g_object_notify(G_OBJECT(self), "station");
-}
-
-static void
-on_engine_notify(GvEngine *engine,
-		 GParamSpec *pspec,
-		 GvPlayer *self)
-{
-	GvPlayerPrivate *priv = self->priv;
 	const gchar *property_name = g_param_spec_get_name(pspec);
+	const PropertyMapping *m;
 
 	TRACE("%p, %s, %p", engine, property_name, self);
 
-	if (!g_strcmp0(property_name, "streaminfo")) {
-		g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_STREAMINFO]);
-
-	} else if (!g_strcmp0(property_name, "metadata")) {
-		g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_METADATA]);
-
-	} else if (!g_strcmp0(property_name, "volume")) {
-		g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_VOLUME]);
-
-	} else if (!g_strcmp0(property_name, "mute")) {
-		g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_MUTE]);
-
-	} else if (!g_strcmp0(property_name, "pipeline-enabled")) {
-		g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_PIPELINE_ENABLED]);
-
-	} else if (!g_strcmp0(property_name, "pipeline-string")) {
-		g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_PIPELINE_STRING]);
-
-	} else if (!g_strcmp0(property_name, "playback-state")) {
-		GvEngineState engine_state;
-		GvPlaybackState playback_state;
-
-		engine_state = gv_engine_get_state(priv->engine);
-
-		/* Map engine state to player state - trivial */
-		switch (engine_state) {
-		case GV_ENGINE_STATE_STOPPED:
-			playback_state = GV_PLAYBACK_STATE_STOPPED;
-			break;
-		case GV_ENGINE_STATE_CONNECTING:
-			playback_state = GV_PLAYBACK_STATE_CONNECTING;
-			break;
-		case GV_ENGINE_STATE_BUFFERING:
-			playback_state = GV_PLAYBACK_STATE_BUFFERING;
-			break;
-		case GV_ENGINE_STATE_PLAYING:
-			playback_state = GV_PLAYBACK_STATE_PLAYING;
-			break;
-		default:
-			ERROR("Unhandled engine state: %d", engine_state);
-			/* Program execution stops here */
-			break;
-		}
-
-		/* Set state */
-		gv_player_set_playback_state(self, playback_state);
+	for (m = engine_mappings; m->name != NULL; m++) {
+		if (g_strcmp0(property_name, m->name) != 0)
+			continue;
+		g_object_notify_by_pspec(G_OBJECT(self), properties[m->id]);
+		break;
 	}
 }
 
-static void
-on_engine_error(GvEngine *engine G_GNUC_UNUSED,
-		const gchar *error_string G_GNUC_UNUSED,
-		GvPlayer *self)
-{
-	/* Whatever the error, just stop */
-	gv_player_stop(self);
-}
-
-static void
-on_engine_ssl_failure(GvEngine *engine G_GNUC_UNUSED,
-		      const gchar *error,
-		      const gchar *debug,
-		      GvPlayer *self)
-{
-	/* Just forward the signal ... */
-	g_signal_emit(self, signals[SIGNAL_SSL_FAILURE], 0, error, debug);
-}
-
 /*
  * Property accessors - construct-only properties
  */
@@ -273,14 +147,23 @@ gv_player_set_engine(GvPlayer *self, GvEngine *engine)
 	GvPlayerPrivate *priv = self->priv;
 
 	/* This is a construct-only property */
-	g_assert_null(priv->engine);
-	g_assert_nonnull(engine);
+	g_assert(priv->engine == NULL);
+	g_assert(engine != NULL);
 	priv->engine = g_object_ref(engine);
 
 	/* Some signal handlers */
 	g_signal_connect_object(engine, "notify", G_CALLBACK(on_engine_notify), self, 0);
-	g_signal_connect_object(engine, "error", G_CALLBACK(on_engine_error), self, 0);
-	g_signal_connect_object(engine, "ssl-failure", G_CALLBACK(on_engine_ssl_failure), self, 0);
+}
+
+static void
+gv_player_set_playback(GvPlayer *self, GvPlayback *playback)
+{
+	GvPlayerPrivate *priv = self->priv;
+
+	/* This is a construct-only property */
+	g_assert(priv->playback == NULL);
+	g_assert(playback != NULL);
+	priv->playback = g_object_ref(playback);
 }
 
 static void
@@ -289,8 +172,8 @@ gv_player_set_station_list(GvPlayer *self, GvStationList *station_list)
 	GvPlayerPrivate *priv = self->priv;
 
 	/* This is a construct-only property */
-	g_assert_null(priv->station_list);
-	g_assert_nonnull(station_list);
+	g_assert(priv->station_list == NULL);
+	g_assert(station_list != NULL);
 	priv->station_list = g_object_ref(station_list);
 }
 
@@ -299,22 +182,6 @@ gv_player_set_station_list(GvPlayer *self, GvStationList *station_list)
  * We don't notify here. It's done in the engine notify handler instead.
  */
 
-GvStreaminfo *
-gv_player_get_streaminfo(GvPlayer *self)
-{
-	GvEngine *engine = self->priv->engine;
-
-	return gv_engine_get_streaminfo(engine);
-}
-
-GvMetadata *
-gv_player_get_metadata(GvPlayer *self)
-{
-	GvEngine *engine = self->priv->engine;
-
-	return gv_engine_get_metadata(engine);
-}
-
 guint
 gv_player_get_volume(GvPlayer *self)
 {
@@ -416,22 +283,22 @@ gv_player_set_pipeline_string(GvPlayer *self, const gchar *pipeline_string)
  * Property accessors - player properties
  */
 
-GvPlaybackState
-gv_player_get_playback_state(GvPlayer *self)
+gboolean
+gv_player_get_playing(GvPlayer *self)
 {
-	return self->priv->state;
+	return self->priv->playing;
 }
 
 static void
-gv_player_set_playback_state(GvPlayer *self, GvPlaybackState state)
+gv_player_set_playing(GvPlayer *self, gboolean playing)
 {
 	GvPlayerPrivate *priv = self->priv;
 
-	if (priv->state == state)
+	if (priv->playing == playing)
 		return;
 
-	priv->state = state;
-	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_PLAYBACK_STATE]);
+	priv->playing = playing;
+	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_PLAYING]);
 }
 
 gboolean
@@ -534,21 +401,24 @@ gv_player_set_station(GvPlayer *self, GvStation *station)
 	if (station == priv->station)
 		return;
 
-	if (priv->station) {
-		g_signal_handlers_disconnect_by_data(priv->station, self);
-		g_object_unref(priv->station);
-		priv->station = NULL;
-	}
+	g_clear_object(&priv->station);
 
-	if (station) {
+	/* Station might not belong to the station list and hence be floating,
+	 * in this case we just want to sink it, we don't want to increase the
+	 * reference counter.
+	 */
+	if (station)
 		priv->station = g_object_ref_sink(station);
-		g_signal_connect_object(priv->station, "notify", G_CALLBACK(on_station_notify), self, 0);
-	}
 
-	g_object_notify(G_OBJECT(self), "station");
-	g_object_notify(G_OBJECT(self), "station-uri");
+	/* Update station for the playback as well */
+	gv_playback_set_station(priv->playback, station);
 
-	INFO("Station set to '%s'", station ? gv_station_get_name_or_uri(station) : NULL);
+	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_STATION]);
+	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_STATION_URI]);
+	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_PREV_STATION]);
+	g_object_notify_by_pspec(G_OBJECT(self), properties[PROP_NEXT_STATION]);
+
+	DEBUG("Station set to '%s'", station ? gv_station_get_name_or_uri(station) : NULL);
 }
 
 gboolean
@@ -564,6 +434,7 @@ gv_player_set_station_by_name(GvPlayer *self, const gchar *name)
 	}
 
 	gv_player_set_station(self, station);
+
 	return TRUE;
 }
 
@@ -581,6 +452,7 @@ gv_player_set_station_by_uri(GvPlayer *self, const gchar *uri)
 	}
 
 	gv_player_set_station(self, station);
+
 	return TRUE;
 }
 
@@ -597,6 +469,7 @@ gv_player_set_station_by_guessing(GvPlayer *self, const gchar *string)
 	}
 
 	gv_player_set_station(self, station);
+
 	return TRUE;
 }
 
@@ -611,12 +484,6 @@ gv_player_get_property(GObject *object,
 	TRACE_GET_PROPERTY(object, property_id, value, pspec);
 
 	switch (property_id) {
-	case PROP_STREAMINFO:
-		g_value_set_boxed(value, gv_player_get_streaminfo(self));
-		break;
-	case PROP_METADATA:
-		g_value_set_boxed(value, gv_player_get_metadata(self));
-		break;
 	case PROP_VOLUME:
 		g_value_set_uint(value, gv_player_get_volume(self));
 		break;
@@ -629,8 +496,8 @@ gv_player_get_property(GObject *object,
 	case PROP_PIPELINE_STRING:
 		g_value_set_string(value, gv_player_get_pipeline_string(self));
 		break;
-	case PROP_PLAYBACK_STATE:
-		g_value_set_enum(value, gv_player_get_playback_state(self));
+	case PROP_PLAYING:
+		g_value_set_boolean(value, gv_player_get_playing(self));
 		break;
 	case PROP_REPEAT:
 		g_value_set_boolean(value, gv_player_get_repeat(self));
@@ -673,6 +540,9 @@ gv_player_set_property(GObject *object,
 	case PROP_ENGINE:
 		gv_player_set_engine(self, g_value_get_object(value));
 		break;
+	case PROP_PLAYBACK:
+		gv_player_set_playback(self, g_value_get_object(value));
+		break;
 	case PROP_STATION_LIST:
 		gv_player_set_station_list(self, g_value_get_object(value));
 		break;
@@ -718,64 +588,36 @@ gv_player_stop(GvPlayer *self)
 {
 	GvPlayerPrivate *priv = self->priv;
 
-	/* To remember what we're doing */
-	priv->wish = GV_PLAYER_WISH_TO_STOP;
-
-	/* Stop playing */
-	gv_engine_stop(priv->engine);
+	gv_player_set_playing(self, FALSE);
+	gv_playback_stop(priv->playback);
 }
 
 void
 gv_player_play(GvPlayer *self)
 {
 	GvPlayerPrivate *priv = self->priv;
-	GvStation *station;
-	GSList *uris;
 
 	/* If no station is set yet, take the first from the station list */
 	if (priv->station == NULL) {
-		GvStation *first_station;
+		GvStation *station;
 
-		first_station = gv_station_list_first(priv->station_list);
-		gv_player_set_station(self, first_station);
+		station = gv_station_list_first(priv->station_list);
+		gv_player_set_station(self, station);
 	}
 
 	/* If there's still no station, return */
-	station = priv->station;
-	if (station == NULL)
+	if (priv->station == NULL)
 		return;
 
-	/* To remember what we're doing */
-	priv->wish = GV_PLAYER_WISH_TO_PLAY;
+	gv_player_set_playing(self, TRUE);
 
-	/* Stop playing */
-	gv_engine_stop(priv->engine);
-
-	/* Get station data */
-	uris = gv_station_get_stream_uris(station);
-
-	/* If there's no URIs, that probably means that the station URI
-	 * points to a playlist, and we need to download it.
-	 */
-	if (uris == NULL) {
-		/* Download the playlist that contains the stream URIs */
-		if (!gv_station_download_playlist(station))
-			WARNING("Can't download playlist");
-
-		/* Downloading a playlist is an asynchronous operation.
-		 * We have nothing left to do here.
-		 */
-		return;
-	} else {
-		/* Play the station */
-		gv_engine_play(priv->engine, station);
-	}
+	/* Start playback */
+	gv_playback_start(priv->playback);
 }
 
 gboolean
 gv_player_next(GvPlayer *self)
 {
-	GvPlayerPrivate *priv = self->priv;
 	GvStation *station;
 
 	station = gv_player_get_next_station(self);
@@ -785,16 +627,12 @@ gv_player_next(GvPlayer *self)
 
 	gv_player_set_station(self, station);
 
-	if (priv->wish == GV_PLAYER_WISH_TO_PLAY)
-		gv_player_play(self);
-
 	return TRUE;
 }
 
 gboolean
 gv_player_prev(GvPlayer *self)
 {
-	GvPlayerPrivate *priv = self->priv;
 	GvStation *station;
 
 	station = gv_player_get_prev_station(self);
@@ -804,9 +642,6 @@ gv_player_prev(GvPlayer *self)
 
 	gv_player_set_station(self, station);
 
-	if (priv->wish == GV_PLAYER_WISH_TO_PLAY)
-		gv_player_play(self);
-
 	return TRUE;
 }
 
@@ -815,18 +650,10 @@ gv_player_toggle(GvPlayer *self)
 {
 	GvPlayerPrivate *priv = self->priv;
 
-	switch (priv->wish) {
-	case GV_PLAYER_WISH_TO_STOP:
-		gv_player_play(self);
-		break;
-	case GV_PLAYER_WISH_TO_PLAY:
+	if (priv->playing == TRUE)
 		gv_player_stop(self);
-		break;
-	default:
-		ERROR("Invalid wish: %d", priv->wish);
-		/* Program execution stops here */
-		break;
-	}
+	else
+		gv_player_play(self);
 }
 
 void
@@ -853,7 +680,7 @@ gv_player_go(GvPlayer *self, const gchar *string_to_play)
 	}
 
 	/* Otherwise, if it's a valid URI, try to play it */
-	if (is_uri_scheme_supported(string_to_play)) {
+	if (gv_is_uri_scheme_supported(string_to_play)) {
 		GvStation *station;
 
 		station = gv_station_new(NULL, string_to_play);
@@ -865,17 +692,15 @@ gv_player_go(GvPlayer *self, const gchar *string_to_play)
 	}
 
 	/* That looks like an invalid string then */
-	WARNING("'%s' is neither a known station or a valid URI", string_to_play);
-	gv_errorable_emit_error(GV_ERRORABLE(self),
-				_("'%s' is neither a known station or a valid URI"),
-				string_to_play);
+	WARNING("Neither a known station nor a valid URI: %s", string_to_play);
 }
 
 GvPlayer *
-gv_player_new(GvEngine *engine, GvStationList *station_list)
+gv_player_new(GvEngine *engine, GvPlayback *playback, GvStationList *station_list)
 {
 	return g_object_new(GV_TYPE_PLAYER,
 			    "engine", engine,
+			    "playback", playback,
 			    "station-list", station_list,
 			    NULL);
 }
@@ -935,6 +760,9 @@ gv_player_finalize(GObject *object)
 	/* Unref the station list */
 	g_object_unref(priv->station_list);
 
+	/* Unref the playback */
+	g_object_unref(priv->playback);
+
 	/* Unref the engine */
 	g_object_unref(priv->engine);
 
@@ -951,8 +779,9 @@ gv_player_constructed(GObject *object)
 	TRACE("%p", object);
 
 	/* Ensure construct-only properties have been set */
-	g_assert_nonnull(priv->engine);
-	g_assert_nonnull(priv->station_list);
+	g_assert(priv->engine != NULL);
+	g_assert(priv->playback != NULL);
+	g_assert(priv->station_list != NULL);
 
 	/* Initialize properties */
 	priv->repeat = DEFAULT_REPEAT;
@@ -994,22 +823,17 @@ gv_player_class_init(GvPlayerClass *class)
 				    GV_TYPE_ENGINE,
 				    GV_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
+	properties[PROP_PLAYBACK] =
+		g_param_spec_object("playback", "Playback", NULL,
+				    GV_TYPE_PLAYBACK,
+				    GV_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
+
 	properties[PROP_STATION_LIST] =
 		g_param_spec_object("station-list", "Station list", NULL,
 				    GV_TYPE_STATION_LIST,
 				    GV_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY);
 
 	/* Engine mirrored properties */
-	properties[PROP_STREAMINFO] =
-		g_param_spec_boxed("streaminfo", "Stream information", NULL,
-				   GV_TYPE_STREAMINFO,
-				   GV_PARAM_READABLE);
-
-	properties[PROP_METADATA] =
-		g_param_spec_boxed("metadata", "Current metadata", NULL,
-				   GV_TYPE_METADATA,
-				   GV_PARAM_READABLE);
-
 	properties[PROP_VOLUME] =
 		g_param_spec_uint("volume", "Volume in percent", NULL,
 				  0, 100, DEFAULT_VOLUME,
@@ -1031,11 +855,10 @@ gv_player_class_init(GvPlayerClass *class)
 				    GV_PARAM_READWRITE);
 
 	/* Player properties */
-	properties[PROP_PLAYBACK_STATE] =
-		g_param_spec_enum("playback-state", "Playback state", NULL,
-				  GV_TYPE_PLAYBACK_STATE,
-				  GV_PLAYBACK_STATE_STOPPED,
-				  GV_PARAM_READABLE);
+	properties[PROP_PLAYING] =
+		g_param_spec_boolean("playing", "Playing", NULL,
+				     FALSE,
+				     GV_PARAM_READABLE);
 
 	properties[PROP_REPEAT] =
 		g_param_spec_boolean("repeat", "Repeat", NULL,
@@ -1074,10 +897,4 @@ gv_player_class_init(GvPlayerClass *class)
 				    GV_PARAM_READABLE);
 
 	g_object_class_install_properties(object_class, PROP_N, properties);
-
-	/* Signals */
-	signals[SIGNAL_SSL_FAILURE] =
-		g_signal_new("ssl-failure", G_TYPE_FROM_CLASS(class),
-			     G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
-			     G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
 }
