@@ -1,7 +1,7 @@
 /*
  * Goodvibes Radio Player
  *
- * Copyright (C) 2020-2021 Arnaud Rebillout
+ * Copyright (C) 2020-2024 Arnaud Rebillout
  *
  * SPDX-License-Identifier: GPL-3.0-only
  *
@@ -23,9 +23,9 @@
  * @title: GvStreaminfo
  * @short_description: Technical information about a stream
  *
- * Streaminfo is a term that appears in the GStreamer documentation,
- * and that was reused here in Goodvibes. So you might want to read
- * the GStreamer documentation first:
+ * Streaminfo is a term that appears in the GStreamer documentation, and that
+ * we reuse here in Goodvibes. So it might be useful to read the GStreamer
+ * documentation first:
  * <https://gstreamer.freedesktop.org/documentation/plugin-development/advanced/tagging.html>
  *
  * Streaminfo are tags that describe the technical parts of stream content.
@@ -42,12 +42,15 @@
  * so. The minimum and maximum bitrate are also updated when a new minimum or a
  * new maximum is reached.
  *
- * There's no data aggregation done in the various update() functions. It means
- * that zero (or NULL) is considered a valid value, rather than "unset". For
- * examples, if we update from GstTags, and the nominal bitrate value is zero
- * in those tags, then we update this value to zero in GvStreaminfo, rather
- * than considering than zero is unset, and not updating the value in
- * GvStreaminfo.
+ * In practice, I'm not always sure how to best handle incoming information
+ * from GStreamer. For example, for a field of type string, when it's NULL, do
+ * we unset whatever we used to know about this field? Or do we consider that
+ * NULL means 'no information', and then skip this field?
+ *
+ * Another example: when GvStreaminfo is updated from GstTags, and the nominal
+ * bitrate value is zero in those tags, what do we do? Answer: we update the
+ * value to zero in GvStreaminfo, therefore considering zero a valid value,
+ * rather than meaning 'no information'. Is it the right thing to do? Maybe.
  */
 
 #include <glib-object.h>
@@ -63,7 +66,7 @@
  */
 
 G_DEFINE_BOXED_TYPE(GvStreaminfo, gv_streaminfo,
-		    gv_streaminfo_ref, gv_streaminfo_unref);
+		    gv_streaminfo_ref, gv_streaminfo_unref)
 
 struct _GvStreaminfo {
 	GvStreamBitrate bitrate;
@@ -119,6 +122,9 @@ gv_streaminfo_update_from_element_setup(GvStreaminfo *self, GstElement *element)
 	GvStreamType type = GV_STREAM_TYPE_UNKNOWN;
 	gboolean changed = FALSE;
 
+	g_return_val_if_fail(self != NULL, FALSE);
+	g_return_val_if_fail(element != NULL, FALSE);
+
 	pads = gst_element_get_pad_template_list(element);
 	for (; pads != NULL; pads = g_list_next(pads)) {
 		GstPadTemplate *pad_template;
@@ -130,7 +136,12 @@ gv_streaminfo_update_from_element_setup(GvStreaminfo *self, GstElement *element)
 			continue;
 
 		caps = gst_pad_template_get_caps(pad_template);
-		// DEBUG("Caps: %s", gst_caps_to_string(caps));  // should be freed
+
+		/* Too noisy */
+		//gchar *caps_str = gst_caps_to_string(caps);
+		//DEBUG("Caps: %s", caps_str);
+		//g_free(caps_str);
+
 		for (i = 0; i < gst_caps_get_size(caps); i++) {
 			GstStructure *structure;
 			const gchar *struct_name;
@@ -164,28 +175,24 @@ gv_streaminfo_update_from_element_setup(GvStreaminfo *self, GstElement *element)
 }
 
 gboolean
-gv_streaminfo_update_from_gst_audio_pad(GvStreaminfo *self, GstPad *audio_pad)
+gv_streaminfo_update_from_gst_caps(GvStreaminfo *self, GstCaps *caps)
 {
-	GstCaps *caps;
-	gint channels;
-	gint sample_rate;
+	GstStructure *s;
+	gchar *caps_str;
+	gint channels = 0;
+	gint sample_rate = 0;
 	gboolean changed = FALSE;
 
 	g_return_val_if_fail(self != NULL, FALSE);
+	g_return_val_if_fail(caps != NULL, FALSE);
 
-	caps = audio_pad ? gst_pad_get_current_caps(audio_pad) : NULL;
+	caps_str = gst_caps_to_string(caps);
+	DEBUG("Caps: %s", caps_str);
+	g_free(caps_str);
 
-	if (caps) {
-		GstStructure *s;
-		// DEBUG("Caps: %s", gst_caps_to_string(caps));  // should be freed
-		s = gst_caps_get_structure(caps, 0);
-		gst_structure_get_int(s, "channels", &channels);
-		gst_structure_get_int(s, "rate", &sample_rate);
-		gst_caps_unref(caps);
-	} else {
-		channels = 0;
-		sample_rate = 0;
-	}
+	s = gst_caps_get_structure(caps, 0);
+	gst_structure_get_int(s, "channels", &channels);
+	gst_structure_get_int(s, "rate", &sample_rate);
 
 	if ((guint) channels != self->channels) {
 		self->channels = channels;
@@ -229,7 +236,12 @@ gv_streaminfo_update_from_gst_taglist(GvStreaminfo *self, GstTagList *taglist)
 	gst_tag_list_get_uint_index(taglist, GST_TAG_MINIMUM_BITRATE, 0, &minimum_bitrate);
 	gst_tag_list_get_uint_index(taglist, GST_TAG_NOMINAL_BITRATE, 0, &nominal_bitrate);
 
-	if (g_strcmp0(codec, self->codec)) {
+	/* If codec is NULL, skip it, do NOT set self->codec to NULL. I have a
+	 * opus stream here, for which we pass by this function twice, and
+	 * codec is NULL the second time. Here's the URL of the stream:
+	 * http://stream.radioreklama.bg/nrj.opus
+	 */
+	if (codec != NULL && g_strcmp0(codec, self->codec) != 0) {
 		g_free(self->codec);
 		self->codec = g_strdup(codec);
 		changed = TRUE;
