@@ -44,6 +44,7 @@ static const gchar *translators =
 	"Daniel Bageac <danielbageac@gmail.com> - English (United States) (en_US)\n"
 	"Adolfo Jayme Barrientos <fitojb@ubuntu.com> - Spanish (es)\n"
 	"Benages <juanjo@benages.eu> - Spanish (es)\n"
+	"Gallegonovato <fran-carro@hotmail.es> - Spanish (es)\n"
 	"Holman Calderón <halecalderon@gmail.com> - Spanish (es)\n"
 	"Jiri Grönroos <jiri.gronroos@iki.fi> - Finnish (fi)\n"
 	"Étienne Deparis <etienne@depar.is> - French (fr)\n"
@@ -64,6 +65,7 @@ static const gchar *translators =
 	"Zbigniew Rosiek <zrosiek@gmail.com> - Polish (pl)\n"
 	"Fúlvio Alves <fga.fulvio@gmail.com> - Portuguese (Brazil) (pt_BR)\n"
 	"Ssantos <ssantos@web.de> - Portuguese (pt)\n"
+	"João Palmeiro <joaommpalmeiro@gmail.com> - Portuguese (Portugal) (pt_PT)\n"
 	"Manuela Silva <mmsrs@sky.com> - Portuguese (Portugal) (pt_PT)\n"
 	"Ssantos <ssantos@web.de> - Portuguese (Portugal) (pt_PT)\n"
 	"Алексей Выскубов <viskubov@gmail.com> - Russian (ru)\n"
