@@ -1,7 +1,7 @@
 /*
  * Goodvibes Radio Player
  *
- * Copyright (C) 2020-2021 Arnaud Rebillout
+ * Copyright (C) 2020-2024 Arnaud Rebillout
  *
  * SPDX-License-Identifier: GPL-3.0-only
  *
@@ -32,6 +32,11 @@
 
 #define UI_RESOURCE_PATH GV_APPLICATION_PATH "/Ui/station-view.glade"
 
+/* Hide playlist streams: I don't see a nice way to make it
+ * fit in the UI, and who cares really, so just drop it.
+ */
+//#define SHOW_PLAYLIST_STREAMS
+
 /*
  * Signal
  */
@@ -51,7 +56,6 @@ static guint signals[SIGNAL_N];
 struct _GvProp {
 	GtkWidget *title_label;
 	GtkWidget *value_label;
-	gboolean show_when_empty;
 };
 
 typedef struct _GvProp GvProp;
@@ -71,10 +75,18 @@ struct _GvStationViewPrivate {
 	GtkWidget *go_back_button;
 	/* Properties */
 	GtkWidget *properties_grid;
+	/* Playback errors */
+	GtkWidget *error_label;
+	GvProp error_message_prop;
+	GvProp error_details_prop;
 	/* Station properties */
 	GtkWidget *stainfo_label;
-	GvProp uri_prop;
-	GvProp streams_prop;
+	GvProp station_uri_prop;
+	GvProp playlist_uri_prop;
+	GvProp playlist_redirection_uri_prop;
+	GvProp playlist_streams_prop;
+	GvProp stream_uri_prop;
+	GvProp stream_redirection_uri_prop;
 	GvProp user_agent_prop;
 	GvProp stream_type_prop;
 	GvProp codec_prop;
@@ -109,7 +121,7 @@ G_DEFINE_TYPE_WITH_PRIVATE(GvStationView, gv_station_view, GTK_TYPE_BOX)
  */
 
 static void
-gv_prop_init(GvProp *prop, GtkBuilder *builder, const gchar *propname, gboolean show_when_empty)
+gv_prop_init(GvProp *prop, GtkBuilder *builder, const gchar *propname)
 {
 	gchar *widget_name;
 	GObject *object;
@@ -138,14 +150,12 @@ gv_prop_init(GvProp *prop, GtkBuilder *builder, const gchar *propname, gboolean
 	gtk_label_set_ellipsize(label, PANGO_ELLIPSIZE_END);
 	gtk_label_set_xalign(label, 0);
 	gtk_label_set_justify(label, GTK_JUSTIFY_LEFT);
-
-	prop->show_when_empty = show_when_empty;
 }
 
 static void
 gv_prop_set(GvProp *prop, const gchar *text)
 {
-	gboolean visible = (text != NULL) || prop->show_when_empty;
+	gboolean visible = (text != NULL);
 
 	gtk_label_set_text(GTK_LABEL(prop->value_label), text);
 	gtk_widget_set_visible(prop->title_label, visible);
@@ -260,6 +270,7 @@ make_sample_rate_string(guint sample_rate)
 	return g_strdup_printf("%g %s", rate, _("kHz"));
 }
 
+#ifdef SHOW_PLAYLIST_STREAMS
 static gchar *
 make_stream_uris_string(GSList *stream_uris)
 {
@@ -283,12 +294,16 @@ make_stream_uris_string(GSList *stream_uris)
 
 	return g_string_free(string, FALSE);
 }
+#endif
 
 static void
-set_station(GvStationViewPrivate *priv, GvStation *station)
+set_station(GvStationViewPrivate *priv, GvStation *station, GvPlayback *playback)
 {
+	GvPlaylist *playlist;
 	const gchar *text;
 
+	// XXX Binding properties would probably work better?
+
 	g_return_if_fail(station != NULL);
 
 	text = gv_station_get_name(station);
@@ -296,29 +311,47 @@ set_station(GvStationViewPrivate *priv, GvStation *station)
 		text = "???";
 	gtk_label_set_text(GTK_LABEL(priv->station_name_label), text);
 
-	text = gv_station_get_uri(station);
-	gv_prop_set(&priv->uri_prop, text);
+	text = gv_playback_get_playlist_uri(playback);
+	gv_prop_set(&priv->playlist_uri_prop, text);
+
+	text = gv_playback_get_playlist_redirection_uri(playback);
+	gv_prop_set(&priv->playlist_redirection_uri_prop, text);
+
+	playlist = gv_playback_get_playlist(playback);
+	if (playlist != NULL) {
+#ifdef SHOW_PLAYLIST_STREAMS
+		GSList *stream_uris;
+		gchar *str;
+		stream_uris = gv_playlist_get_stream_uris(playlist);
+		str = make_stream_uris_string(stream_uris);
+		gv_prop_set(&priv->playlist_streams_prop, str);
+		g_free(str);
+#else
+		gv_prop_set(&priv->playlist_streams_prop, NULL);
+#endif
+	} else {
+		gv_prop_set(&priv->playlist_streams_prop, NULL);
+	}
+
+	text = gv_playback_get_stream_uri(playback);
+	gv_prop_set(&priv->stream_uri_prop, text);
+
+	text = gv_playback_get_stream_redirection_uri(playback);
+	gv_prop_set(&priv->stream_redirection_uri_prop, text);
 
 	text = gv_station_get_user_agent(station);
 	gv_prop_set(&priv->user_agent_prop, text);
 
-	text = gv_station_get_first_stream_uri(station);
-	if (text == NULL) {
-		/* There's no stream uris to display */
-		gv_prop_set(&priv->streams_prop, NULL);
-	} else if (!g_strcmp0(text, gv_station_get_uri(station))) {
-		/* If station uri and stream uri are the same, there's
-		 * no need to display it. */
-		gv_prop_set(&priv->streams_prop, NULL);
+	/* We show the station uri as defined by user only when the station is
+	 * not playing: at this stage, we don't know yet if this uri is for a
+	 * playlist or a stream. However when the station is playing, we
+	 * already show this uri, either as playlist uri, either as stream uri.
+	 */
+	if (playlist == NULL && gv_playback_get_stream_uri(playback) == NULL) {
+		text = gv_station_get_uri(station);
+		gv_prop_set(&priv->station_uri_prop, text);
 	} else {
-		/* Let's display the list of stream uris then */
-		GSList *stream_uris;
-		gchar *str;
-
-		stream_uris = gv_station_get_stream_uris(station);
-		str = make_stream_uris_string(stream_uris);
-		gv_prop_set(&priv->streams_prop, str);
-		g_free(str);
+		gv_prop_set(&priv->station_uri_prop, NULL);
 	}
 }
 
@@ -327,9 +360,13 @@ unset_station(GvStationViewPrivate *priv)
 {
 	gtk_label_set_text(GTK_LABEL(priv->station_name_label),
 			   _("No station selected"));
-	gv_prop_set(&priv->uri_prop, NULL);
+	gv_prop_set(&priv->station_uri_prop, NULL);
+	gv_prop_set(&priv->playlist_uri_prop, NULL);
+	gv_prop_set(&priv->playlist_redirection_uri_prop, NULL);
+	gv_prop_set(&priv->playlist_streams_prop, NULL);
+	gv_prop_set(&priv->stream_uri_prop, NULL);
+	gv_prop_set(&priv->stream_redirection_uri_prop, NULL);
 	gv_prop_set(&priv->user_agent_prop, NULL);
-	gv_prop_set(&priv->streams_prop, NULL);
 }
 
 static void
@@ -342,6 +379,21 @@ set_playback_status(GvStationViewPrivate *priv, GvPlaybackState state)
 	gtk_label_set_text(label, playback_state_str);
 }
 
+static void
+set_playback_error(GvStationViewPrivate *priv, const gchar *message,
+		const gchar *details)
+{
+	gv_prop_set(&priv->error_message_prop, message);
+	gv_prop_set(&priv->error_details_prop, details);
+}
+
+static void
+unset_playback_error(GvStationViewPrivate *priv)
+{
+	gv_prop_set(&priv->error_message_prop, NULL);
+	gv_prop_set(&priv->error_details_prop, NULL);
+}
+
 static void
 set_streaminfo(GvStationViewPrivate *priv, GvStreaminfo *streaminfo)
 {
@@ -423,31 +475,46 @@ unset_metadata(GvStationViewPrivate *priv)
  */
 
 static void
-gv_station_view_update_station(GvStationView *self, GvPlayer *player)
+gv_station_view_update_station(GvStationView *self, GvPlayback *playback)
 {
 	GvStationViewPrivate *priv = self->priv;
-	GvStation *station = gv_player_get_station(player);
+	GvStation *station = gv_playback_get_station(playback);
 
 	if (station)
-		set_station(priv, station);
+		set_station(priv, station, playback);
 	else
 		unset_station(priv);
 }
 
 static void
-gv_station_view_update_playback_status(GvStationView *self, GvPlayer *player)
+gv_station_view_update_playback_status(GvStationView *self, GvPlayback *playback)
 {
 	GvStationViewPrivate *priv = self->priv;
-	GvPlaybackState state = gv_player_get_playback_state(player);
+	GvPlaybackState state = gv_playback_get_state(playback);
 
 	set_playback_status(priv, state);
 }
 
 static void
-gv_station_view_update_streaminfo(GvStationView *self, GvPlayer *player)
+gv_station_view_update_playback_error(GvStationView *self, GvPlayback *playback)
 {
 	GvStationViewPrivate *priv = self->priv;
-	GvStreaminfo *streaminfo = gv_player_get_streaminfo(player);
+	GvPlaybackError *error = gv_playback_get_error(playback);
+
+	if (error != NULL) {
+		set_playback_error(priv, error->message, error->details);
+		gtk_widget_set_visible(priv->error_label, TRUE);
+	} else {
+		unset_playback_error(priv);
+		gtk_widget_set_visible(priv->error_label, FALSE);
+	}
+}
+
+static void
+gv_station_view_update_streaminfo(GvStationView *self, GvPlayback *playback)
+{
+	GvStationViewPrivate *priv = self->priv;
+	GvStreaminfo *streaminfo = gv_playback_get_streaminfo(playback);
 
 	if (streaminfo)
 		set_streaminfo(priv, streaminfo);
@@ -456,10 +523,10 @@ gv_station_view_update_streaminfo(GvStationView *self, GvPlayer *player)
 }
 
 static void
-gv_station_view_update_metadata(GvStationView *self, GvPlayer *player)
+gv_station_view_update_metadata(GvStationView *self, GvPlayback *playback)
 {
 	GvStationViewPrivate *priv = self->priv;
-	GvMetadata *metadata = gv_player_get_metadata(player);
+	GvMetadata *metadata = gv_playback_get_metadata(playback);
 
 	if (metadata) {
 		set_metadata(priv, metadata);
@@ -475,21 +542,25 @@ gv_station_view_update_metadata(GvStationView *self, GvPlayer *player)
  */
 
 static void
-on_player_notify(GvPlayer *player, GParamSpec *pspec,
-		 GvStationView *self)
+on_playback_notify(GvPlayback *playback, GParamSpec *pspec, GvStationView *self)
 {
 	const gchar *property_name = g_param_spec_get_name(pspec);
 
-	TRACE("%p, %s, %p", player, property_name, self);
+	TRACE("%p, %s, %p", playback, property_name, self);
 
-	if (!g_strcmp0(property_name, "station"))
-		gv_station_view_update_station(self, player);
-	else if (!g_strcmp0(property_name, "playback-state"))
-		gv_station_view_update_playback_status(self, player);
+	if (!g_strcmp0(property_name, "playlist-uri") ||
+	    !g_strcmp0(property_name, "playlist-redirection-uri") ||
+	    !g_strcmp0(property_name, "stream-uri") ||
+	    !g_strcmp0(property_name, "stream-redirection-uri"))
+		gv_station_view_update_station(self, playback);
+	else if (!g_strcmp0(property_name, "state"))
+		gv_station_view_update_playback_status(self, playback);
+	else if (!g_strcmp0(property_name, "error"))
+		gv_station_view_update_playback_error(self, playback);
 	else if (!g_strcmp0(property_name, "streaminfo"))
-		gv_station_view_update_streaminfo(self, player);
+		gv_station_view_update_streaminfo(self, playback);
 	else if (!g_strcmp0(property_name, "metadata"))
-		gv_station_view_update_metadata(self, player);
+		gv_station_view_update_metadata(self, playback);
 }
 
 static void
@@ -501,27 +572,31 @@ on_go_back_button_clicked(GtkButton *button G_GNUC_UNUSED, GvStationView *self)
 static void
 on_map(GvStationView *self, gpointer user_data)
 {
-	GvPlayer *player = gv_core_player;
+	GvPlayback *playback = gv_core_playback;
 
 	TRACE("%p, %p", self, user_data);
 
-	g_signal_connect_object(player, "notify",
-				G_CALLBACK(on_player_notify), self, 0);
+	/* Connect core signal handlers */
+	g_signal_connect_object(playback, "notify",
+				G_CALLBACK(on_playback_notify), self, 0);
 
-	gv_station_view_update_station(self, player);
-	gv_station_view_update_playback_status(self, player);
-	gv_station_view_update_streaminfo(self, player);
-	gv_station_view_update_metadata(self, player);
+	/* Update widgets */
+	gv_station_view_update_station(self, playback);
+	gv_station_view_update_playback_status(self, playback);
+	gv_station_view_update_playback_error(self, playback);
+	gv_station_view_update_streaminfo(self, playback);
+	gv_station_view_update_metadata(self, playback);
 }
 
 static void
 on_unmap(GvStationView *self, gpointer user_data G_GNUC_UNUSED)
 {
-	GvPlayer *player = gv_core_player;
+	GvPlayback *playback = gv_core_playback;
 
 	TRACE("%p, %p", self, user_data);
 
-	g_signal_handlers_disconnect_by_data(player, self);
+	/* Disconnect core signal handlers */
+	g_signal_handlers_disconnect_by_data(playback, self);
 }
 
 /*
@@ -561,25 +636,34 @@ gv_station_view_populate_widgets(GvStationView *self)
 	/* Properties */
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, properties_grid);
 
+	/* Playback errors */
+	GTK_BUILDER_SAVE_WIDGET(builder, priv, error_label);
+	gv_prop_init(&priv->error_message_prop, builder, "error_message");
+	gv_prop_init(&priv->error_details_prop, builder, "error_details");
+
 	/* Station Properties */
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, stainfo_label);
-	gv_prop_init(&priv->uri_prop, builder, "uri", TRUE);
-	gv_prop_init(&priv->streams_prop, builder, "streams", FALSE);
-	gv_prop_init(&priv->user_agent_prop, builder, "user_agent", FALSE);
-	gv_prop_init(&priv->stream_type_prop, builder, "stream_type", FALSE);
-	gv_prop_init(&priv->codec_prop, builder, "codec", FALSE);
-	gv_prop_init(&priv->channels_prop, builder, "channels", FALSE);
-	gv_prop_init(&priv->sample_rate_prop, builder, "sample_rate", FALSE);
-	gv_prop_init(&priv->bitrate_prop, builder, "bitrate", FALSE);
+	gv_prop_init(&priv->station_uri_prop, builder, "station_uri");
+	gv_prop_init(&priv->playlist_uri_prop, builder, "playlist_uri");
+	gv_prop_init(&priv->playlist_redirection_uri_prop, builder, "playlist_redirection_uri");
+	gv_prop_init(&priv->playlist_streams_prop, builder, "playlist_streams");
+	gv_prop_init(&priv->stream_uri_prop, builder, "stream_uri");
+	gv_prop_init(&priv->stream_redirection_uri_prop, builder, "stream_redirection_uri");
+	gv_prop_init(&priv->user_agent_prop, builder, "user_agent");
+	gv_prop_init(&priv->stream_type_prop, builder, "stream_type");
+	gv_prop_init(&priv->codec_prop, builder, "codec");
+	gv_prop_init(&priv->channels_prop, builder, "channels");
+	gv_prop_init(&priv->sample_rate_prop, builder, "sample_rate");
+	gv_prop_init(&priv->bitrate_prop, builder, "bitrate");
 
 	/* Metadata */
 	GTK_BUILDER_SAVE_WIDGET(builder, priv, metadata_label);
-	gv_prop_init(&priv->title_prop, builder, "title", FALSE);
-	gv_prop_init(&priv->artist_prop, builder, "artist", FALSE);
-	gv_prop_init(&priv->album_prop, builder, "album", FALSE);
-	gv_prop_init(&priv->genre_prop, builder, "genre", FALSE);
-	gv_prop_init(&priv->year_prop, builder, "year", FALSE);
-	gv_prop_init(&priv->comment_prop, builder, "comment", FALSE);
+	gv_prop_init(&priv->title_prop, builder, "title");
+	gv_prop_init(&priv->artist_prop, builder, "artist");
+	gv_prop_init(&priv->album_prop, builder, "album");
+	gv_prop_init(&priv->genre_prop, builder, "genre");
+	gv_prop_init(&priv->year_prop, builder, "year");
+	gv_prop_init(&priv->comment_prop, builder, "comment");
 
 	/* Pack that within the box */
 	gtk_container_add(GTK_CONTAINER(self), priv->station_view_box);
@@ -607,6 +691,8 @@ gv_station_view_setup_appearance(GvStationView *self)
 		     "margin-end", GV_UI_WINDOW_MARGIN,
 		     "margin-bottom", GV_UI_WINDOW_MARGIN,
 		     NULL);
+
+	gtk_label_set_xalign(GTK_LABEL(priv->error_label), 1.0);
 	gtk_label_set_xalign(GTK_LABEL(priv->stainfo_label), 1.0);
 	gtk_label_set_xalign(GTK_LABEL(priv->metadata_label), 1.0);
 }
@@ -616,12 +702,13 @@ gv_station_view_setup_widgets(GvStationView *self)
 {
 	GvStationViewPrivate *priv = self->priv;
 
-	g_signal_connect_object(self, "map",
-				G_CALLBACK(on_map), NULL, 0);
-	g_signal_connect_object(self, "unmap",
-				G_CALLBACK(on_unmap), NULL, 0);
+	/* Connect widgets signal handlers */
 	g_signal_connect_object(priv->go_back_button, "clicked",
 				G_CALLBACK(on_go_back_button_clicked), self, 0);
+
+	/* Connect self signal handlers */
+	g_signal_connect_object(self, "map", G_CALLBACK(on_map), NULL, 0);
+	g_signal_connect_object(self, "unmap", G_CALLBACK(on_unmap), NULL, 0);
 }
 
 /*
@@ -675,7 +762,7 @@ gv_station_view_class_init(GvStationViewClass *class)
 
 	/* Signals */
 	signals[SIGNAL_GO_BACK_CLICKED] =
-		g_signal_new("go-back-clicked", G_TYPE_FROM_CLASS(class),
+		g_signal_new("go-back-clicked", G_OBJECT_CLASS_TYPE(class),
 			     G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
 			     G_TYPE_NONE, 0);
 }
